// 3DRotate.cpp
// ported to Windows CE by Supareak Mulapong (ng@jimmy.com)

#include <windows.h>
#include <math.h>

#define NUM_FACE    14
#define NUM_VECT    16
#define NUM_POINT    4
#define NUM_STAR   200
#define SCREENCX   200
#define SCREENCY   200

WCHAR     szAppName[] = L"3DROT";
WCHAR     szTitle[] = L"3D Rotate";
HWND      hWndMain;
HINSTANCE hInst;

HDC       hdcv;
HBITMAP   hbmpv;
BYTE     *dst;
HBRUSH    brush[64];

float     cosine[360];
float     sine[360];

typedef struct
{
  float x,y,z;
} VECT;

typedef struct
{
  int x,y,z;
} STAR;

VECT object[]=
{
  {-10,30,10,},{-30,10,30},{-30,-10,30},{-10,-30,10},
  {10,-30,10},{30,-10,30},{30,10,30},{10,30,10},
  {10,-30,-10},{30,-10,-30},{30,10,-30},{10,30,-10},
  {-10,-30,-10},{-30,-10,-30},{-30,10,-30},{-10,30,-10}
};

VECT  translate[NUM_VECT];
VECT  projected[NUM_VECT];
VECT  unit_vector[NUM_FACE];
VECT  light={-1,-1,-1};
STAR  stars[NUM_STAR];

int face[NUM_FACE][NUM_POINT]=
{
  {0,1,6,7},{1,2,5,6},{2,3,4,5},{7,6,10,11},
  {6,5,9,10},{5,4,8,9},{11,10,14,15},{10,9,13,14},
  {9,8,12,13},{15,14,1,0},{14,13,2,1},{13,12,3,2},
  {15,0,7,11},{3,12,8,4}
};

int xangle=0;
int yangle=0;
int zangle=0;
int obj_xoff=100;
int obj_yoff=100;
int obj_zoff=2000;

void create_lookup_tables(void)
{
  int i;

  for(i=0;i<360;i++)
  {
    cosine[i] = (float)cos((i*3.14159265)/180);
    sine[i]   = (float)sin((i*3.14159265)/180);
  }

  for(i=0; i<64; i++)
    brush[i] = CreateSolidBrush(RGB(32+i*3,0,0));
}

void calc_unitvect(void)
{
  int i, normal_len;
  int u1,u2,u3,v1,v2,v3;
  VECT normal_vect;

  for(i=0; i<NUM_FACE; i++)
  {
    u1 = (int)(projected[face[i][1]].x - projected[face[i][0]].x);
    u2 = (int)(projected[face[i][1]].y - projected[face[i][0]].y);
    u3 = (int)(projected[face[i][1]].z - projected[face[i][0]].z);

    v1 = (int)(projected[face[i][2]].x-projected[face[i][0]].x);
    v2 = (int)(projected[face[i][2]].y-projected[face[i][0]].y);
    v3 = (int)(projected[face[i][2]].z-projected[face[i][0]].z);

    normal_vect.x = (float)(u2*v3 - u3*v2);
    normal_vect.y = (float)(u3*v1 - u1*v3);
    normal_vect.z = (float)(u1*v2 - u2*v1);

    normal_len = (int)sqrt(normal_vect.x*normal_vect.x + normal_vect.y*normal_vect.y + normal_vect.z*normal_vect.z);

    unit_vector[i].x = normal_vect.x/normal_len;
    unit_vector[i].y = normal_vect.y/normal_len;
    unit_vector[i].z = normal_vect.z/normal_len;
  }
}

void rotate_object(void)
{
  int i;
  VECT temp;

  for(i=0; i<NUM_VECT; i++)
  {
    /*rotate around x-axis*/
    temp.x = object[i].x;
    temp.y = object[i].y*cosine[xangle] - object[i].z*sine[xangle];
    temp.z = object[i].y*sine[xangle] + object[i].z*cosine[xangle];
    translate[i] = temp;

    /*rotate around y-axis*/
    temp.x = translate[i].x*cosine[yangle] - translate[i].z*sine[yangle];
    temp.y = translate[i].y;
    temp.z = translate[i].x*sine[yangle] + translate[i].z*cosine[yangle];
    translate[i] = temp;

    /*rotate around z-axis*/
    temp.x = translate[i].x*cosine[zangle] - translate[i].y*sine[zangle];
    temp.y = translate[i].x*sine[zangle] + translate[i].y*cosine[zangle];
    temp.z = translate[i].z;
    translate[i] = temp;
  }
}

void project_object(void)
{
  int i;

  for(i=0; i<NUM_VECT; i++)
  {
    projected[i].x = (translate[i].x * 400) / (translate[i].z + obj_zoff) + obj_xoff;
    projected[i].y = (translate[i].y * 400) / (translate[i].z + obj_zoff) + obj_yoff;
    projected[i].z = translate[i].z;
  }
}

void render_object(HDC hDC)
{
  int   i;
  int   intensity;
  POINT poly[4];
  HPEN  oldpen = (HPEN)SelectObject(hDC,(HPEN)GetStockObject(NULL_PEN));
  HBRUSH oldbr = (HBRUSH)SelectObject(hDC,(HBRUSH)GetStockObject(BLACK_BRUSH));

  for(i=0; i<NUM_FACE; i++)
  {
    if(unit_vector[i].z <= 0)
    {
      poly[0].x = (int)projected[face[i][0]].x;
      poly[0].y = (int)projected[face[i][0]].y;
      poly[1].x = (int)projected[face[i][1]].x;
      poly[1].y = (int)projected[face[i][1]].y;
      poly[2].x = (int)projected[face[i][2]].x;
      poly[2].y = (int)projected[face[i][2]].y;
      poly[3].x = (int)projected[face[i][3]].x;
      poly[3].y = (int)projected[face[i][3]].y;

      intensity = (int)((unit_vector[i].x*light.x + unit_vector[i].y*light.y + unit_vector[i].z*light.z) * 64);
      intensity = max(0,min(63,intensity));
      SelectObject(hDC,brush[intensity]);
      Polygon(hDC,poly,4);

      if(obj_zoff > 200)
        obj_zoff -= 1;
    }
  }

  SelectObject(hDC,oldpen);
  SelectObject(hDC,oldbr);
}

void init_stars(void)
{
  int loop;

  for(loop=0; loop<NUM_STAR; loop++)
  {
    stars[loop].x = (Random()%SCREENCX)-(SCREENCX/2);
    stars[loop].y = (Random()%SCREENCY)-(SCREENCY/2);
    stars[loop].z = loop+1;
  }
}

void SetDstPixel(int x, int y, BYTE pixel)
{
  int lw;

  if(x<0 || x>=SCREENCX || y<0 || y>=SCREENCY)
    return;

  lw = (SCREENCX+3)&~3;
  *(dst+(SCREENCY-y-1)*lw+x) = pixel + 64;;
}

void do_stars(void)
{
  int loop;
  BYTE c;
  POINT new_pos;

  for(loop=0; loop<NUM_STAR; loop++)
  {
    new_pos.x = (int)((stars[loop].x*400)/stars[loop].z) + obj_xoff;
    new_pos.y = (int)((stars[loop].y*400)/stars[loop].z) + obj_yoff;

    if((new_pos.x>0) && (new_pos.x<200) && (new_pos.y>0) && (new_pos.y<200))
    {
      if(stars[loop].z>512)
        c = 0;
      else
        c = 255-(BYTE)(stars[loop].z/2);

      SetDstPixel(new_pos.x,new_pos.y,c/4);
    }

    stars[loop].z -= 10;

    if(stars[loop].z<1)
      stars[loop].z += NUM_STAR;
  }
}

//
//----------------- WINDOWS INTERFACE ------------------------------
//

LRESULT CALLBACK WndProc ( HWND hWnd,UINT message,WPARAM uParam,LPARAM lParam)
{
  HDC hdc;

  static int brecall = 0;

  switch (message)
  {
  case WM_TIMER:
    if(brecall) break;

    brecall = 1;

    if(hdc=GetDC(hWnd))
    {
      PatBlt(hdcv,0,0,SCREENCX,SCREENCY,BLACKNESS);
      xangle = (xangle+5) % 360;
      yangle = (yangle+5) % 360;
      zangle = (zangle+10) % 360;
      do_stars();
      rotate_object();
      project_object();
      calc_unitvect();
      render_object(hdcv);
      ExtTextOut(hdcv,0,0,0,0,L"jimmy.com",9,0);
      BitBlt(hdc,0,0,SCREENCX,SCREENCY,hdcv,0,0,SRCCOPY);
      ReleaseDC(hWnd,hdc);
    }

    brecall = 0;
    break;

  case WM_LBUTTONDOWN:
  case WM_KEYDOWN:
    DestroyWindow(hWnd);
    break;

  case WM_DESTROY:
    PostQuitMessage(0);
    break;

  default:
    return DefWindowProc(hWnd, message, uParam, lParam);
  }

  return (0);
}

BOOL InitApplication (HINSTANCE hInstance)
{
  WNDCLASSW wc;

  wc.style = 0 ;
  wc.lpfnWndProc = (WNDPROC) WndProc;
  wc.cbClsExtra = 0;
  wc.cbWndExtra = 0;
  wc.hInstance = hInstance;
  wc.hIcon = LoadIcon(hInstance,MAKEINTRESOURCE(1));
  wc.hCursor = NULL;
  wc.hbrBackground = (HBRUSH) GetStockObject(BLACK_BRUSH);
  wc.lpszMenuName = NULL;
  wc.lpszClassName = szAppName;

  return (BOOL) RegisterClass(&wc);
}

void SetWindowToCenter(HWND hwnd)
{
  RECT rc;
  int cx, cy, wx, wy;
  
  cx = GetSystemMetrics(SM_CXSCREEN);
  cy = GetSystemMetrics(SM_CYSCREEN)-26;
  GetWindowRect(hwnd,&rc);
  wx = rc.right - rc.left;
  wy = rc.bottom - rc.top;
  rc.left = (cx-wx)/2;
  rc.top = (cy-wy)/2;
  MoveWindow(hwnd,rc.left,rc.top,wx,wy,FALSE);
}

HBITMAP CreateImage(int w, int h, BYTE **pbitaddress)
{
  LPBITMAPINFO pdib = NULL;
  LPBITMAPINFOHEADER phdr;
  int dibsize;
  int linew;
  HDC hDC;
  HBITMAP hDib;

  linew = (w+3)&~3;
  dibsize = sizeof(BITMAPINFOHEADER) + (256 * sizeof(RGBQUAD));
  hDC = GetDC(NULL);

  if(pdib = (LPBITMAPINFO)LocalAlloc(LPTR,dibsize))
  {
    int i;

    phdr = &pdib->bmiHeader;
    phdr->biSize = sizeof(BITMAPINFOHEADER);
    phdr->biWidth = w;
    phdr->biHeight = h;
    phdr->biPlanes = 1;
    phdr->biBitCount = 8;
    phdr->biCompression = BI_RGB;
    phdr->biSizeImage = dibsize + (linew * h);
    phdr->biXPelsPerMeter = 72;
    phdr->biYPelsPerMeter = 72;
    phdr->biClrUsed = 256;
    phdr->biClrImportant = 0;

    // color palette for rotate object
    for(i=0; i<64; i++)
    {
      pdib->bmiColors[i].rgbBlue = 0;
      pdib->bmiColors[i].rgbRed = 32+i*3;
      pdib->bmiColors[i].rgbGreen = 0;
      pdib->bmiColors[i].rgbReserved = 0;
    }

    // color palette for star
    for(; i<128; i++)
    {
      pdib->bmiColors[i].rgbBlue = (i-64)*4;
      pdib->bmiColors[i].rgbRed = (i-64)*4;
      pdib->bmiColors[i].rgbGreen = (i-64)*4;
      pdib->bmiColors[i].rgbReserved = 0;
    }

    for(; i<256; i++)
    {
      pdib->bmiColors[i].rgbBlue = 0;
      pdib->bmiColors[i].rgbRed = 0;
      pdib->bmiColors[i].rgbGreen = 0;
      pdib->bmiColors[i].rgbReserved = 0;
    }
  }

  hDib = CreateDIBSection(hDC, pdib, DIB_RGB_COLORS, (void**)pbitaddress, NULL, NULL);
  ReleaseDC(NULL,hDC);
  LocalFree((HLOCAL)pdib);

  return hDib;
}

BOOL InitInstance(HINSTANCE hInstance, int nCmdShow)
{
  HWND hWnd;
  HDC hdc;

  hInst = hInstance;

  // Use the default window settings.
  hWnd = CreateWindow(szAppName,szTitle,WS_BORDER,
        0,0,SCREENCX,SCREENCY,
        NULL, NULL, hInstance, NULL);

  if (hWnd == 0)    // Check whether values returned by CreateWindow() are valid.
    return (FALSE);

  if (IsWindow(hWnd) != TRUE)
    return (FALSE);

  hWndMain = hWnd;

  hdc = GetDC(hWndMain);
  hdcv = CreateCompatibleDC(hdc);
  hbmpv = CreateImage(SCREENCX,SCREENCY,&dst);
  SelectObject(hdcv,hbmpv);
  ReleaseDC(hWndMain,hdc);
  init_stars();
  create_lookup_tables();

  SetWindowToCenter(hWndMain);
  ShowWindow(hWnd, SW_SHOW);
  UpdateWindow(hWnd);
  SetTimer(hWnd,1,50,NULL);

  return(TRUE);
}

int WINAPI WinMain (HINSTANCE hInstance,HINSTANCE hPrevInstance,LPWSTR lpCmdLine,int nCmdShow)
{
  MSG msg;

  if ( msg.hwnd = FindWindow(szAppName,NULL) )
  {
    ShowWindow(msg.hwnd,SW_SHOW);
    return FALSE;
  }

  if (InitApplication(hInstance) == FALSE)
    return(FALSE);

  if (InitInstance(hInstance, nCmdShow) == FALSE)
    return(FALSE);

  while (GetMessage(&msg, NULL, 0, 0) == TRUE)
    DispatchMessage(&msg);

  DeleteDC(hdcv);
  DeleteObject(hbmpv);

  return(msg.wParam);
}
