// mamece.c : Defines the entry point for the application.
//

#include <windows.h>
#include <commctrl.h>
#include <stdarg.h>

#include "cgamex.h"
#include "osdepend.h"
#include "mame.h"
#include "driver.h"
#include "mamece.h"
#include "resource.h"
#include "display.h"
#include "joystick.h"
#include "trak.h"
#include "keyboard.h"
#include "sound.h"
#include "file.h"

#define MAX_LOADSTRING 100

#define IDC_PLAYLIST    200
#define IDC_PLAY        201
#define IDC_OPTIONS     202

/***************************************************************************
    Function prototypes
 ***************************************************************************/

static LRESULT CALLBACK     MAME32_MessageProc(HWND, UINT, WPARAM, LPARAM);
static HWND                 MAME32_CreateWindow(void);
static void                 MAME32_ProcessMessages(void);
static BOOL                 MAME32_PumpAndReturnMessage(MSG* pMsg);
static void                 MAME32_Quit(void);
static BOOL                 MAME32_DetectMMX();
static BOOL                 MAME32_Done(void);
static BOOL                 MAME32_Pause(void);

/***************************************************************************
    External variables
 ***************************************************************************/

static char szRootPath[MAX_PATH];

struct tMAME32App   MAME32App = 
{
    { NULL },                   /* m_hInstance       */
    { NULL },                   /* m_hwndUI          */
    { NULL },                   /* m_hWnd            */
    { "MAME32" },               /* m_pName           */
	{ szRootPath },				/* m_pRootPath		 */
    { FALSE },                  /* m_bIsInitialized  */
    { FALSE },                  /* m_bIsActive       */
    { FALSE },                  /* m_bIsPaused       */
    { FALSE },                  /* m_bDone           */
    { FALSE },                  /* m_bPaused         */
    { FALSE },                  /* m_bUseAIMouse     */

    { 0 },	                    /* m_pDisplay        */
    { NULL },                   /* m_pDisplay        */
    { NULL },                   /* m_pSound          */
    { NULL },                   /* m_pKeyboard       */
    { NULL },                   /* m_pJoystick       */
    { NULL },                   /* m_pTrak           */
    { NULL },                   /* m_pFMSynth        */
#ifdef MAME_NET
    { NULL },                   /* m_pNetwork        */
    { FALSE },                  /* m_bUseNetwork     */
#endif /* MAME_NET */

    { MAME32_CreateWindow },            /* CreateMAMEWindow     */
    { MAME32_ProcessMessages },         /* ProcessMessages      */
    { MAME32_PumpAndReturnMessage },    /* PumpAndReturnMessage */
    { MAME32_Quit },                    /* Quit                 */
    { MAME32_Done },                    /* close was clicked    */
    { MAME32_Pause }                    /* auto_pause key cache */
};

// Global Variables:
HINSTANCE			hInst;			// The current instance
HWND				hwndCB;			// The command bar handle

// Foward declarations of functions included in this code module:


/***************************************************************************
    Internal structures
 ***************************************************************************/

/***************************************************************************
    Internal variables
 ***************************************************************************/

/* our dialog/configured options */
static options_type playing_game_options;

static game_data_type*  game_data;			/* Game data for all games */
static int              game_count;
static int*				game_index;			/* Index of available games */
static int				game_index_count;	
static int              play_game;
static BOOL				enable_sound;		/* Enable sound in games */
static BOOL				show_profiler;		/* Display the profiler */
static BOOL				show_framerate;		/* Display the profiler */
static BOOL				show_all_games;		/* Should we show all games? */	
static BOOL				show_clones;		/* Don't show clone games */
static BOOL				gfxopt_flip;		/* Optimize flipped graphics */
static BOOL				gfxopt_align;		/* Use aligned bitmaps */
static BOOL				gfxopt_pens;		/* Optimize pens */
static HWND				hGameList;

/***************************************************************************
    Internal functions
 ***************************************************************************/

static void RefreshGameListBox();
static void MamePlayGame(int index);

/***************************************************************************
    External functions  
 ***************************************************************************/

void MAME32App_init(options_type *options)
{
    MAME32App.m_hWnd = NULL;

    MAME32App.m_bIsInitialized = FALSE;
    MAME32App.m_bIsActive      = FALSE;
    MAME32App.m_bIsPaused      = FALSE;
    MAME32App.m_pDisplay    = NULL;
    MAME32App.m_pSound      = NULL;
    MAME32App.m_pKeyboard   = NULL;
    MAME32App.m_pJoystick   = NULL;
    MAME32App.m_pTrak       = NULL;
    MAME32App.m_pFMSynth    = NULL;
    MAME32App.m_bDone       = FALSE;

    /*
        Machine->scrbitmap is not initialized in the mame source
        until after the call to osd_create_display().
        This causes a problem if osd_create_display() detects an error,
        then shows the error message box, which causes focus to change,
        which causes OnPause() to be called, which uses Machine->scrbitmap.
        Whew. So it needs to be set to NULL so it doesn't crash in OnPause().
    */
    Machine->scrbitmap = NULL;
}

TCHAR *WC(const char *str)
{
	static WORD wide[1024];
	
	MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, str, strlen(str) + 1, wide, 1023);

	return wide;
}

TCHAR *WC2(const char *str)
{
	static WORD wide[1024];
	
	MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, str, strlen(str) + 1, wide, 1023);

	return wide;
}

TCHAR *WC3(const char *str)
{
	static WORD wide[1024];
	
	MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, str, strlen(str) + 1, wide, 1023);

	return wide;
}

void ErrorMsg(const char* fmt, ...)
{
	char buf[512];
	va_list ptr;

	va_start(ptr, fmt);
	vsprintf(buf, fmt, ptr);

	MessageBox(NULL, WC(buf), WC2("ERROR"), MB_ICONSTOP | MB_OK);
}

void _assert(int exp)
{
	MessageBox(0, _T("Assertion failure detected"), _T("ASSERT"), MB_ICONSTOP | MB_OK);
}

/***************************************************************************
    Internal functions
 ***************************************************************************/

static HWND MAME32_CreateWindow(void)
{
    static BOOL     bRegistered = FALSE;
    HINSTANCE       hInstance = GetModuleHandle(NULL);
	HWND			hWnd;
	WNDCLASS		wc;

    if (bRegistered == FALSE)
    {
		wc.style			= CS_HREDRAW | CS_VREDRAW;
		wc.lpfnWndProc		= (WNDPROC) MAME32_MessageProc;
		wc.cbClsExtra		= 0;
		wc.cbWndExtra		= 0;
		wc.hInstance		= hInstance;
		wc.hIcon			= LoadIcon(hInstance, MAKEINTRESOURCE(IDI_MAMECE));
		wc.hCursor			= 0;
		wc.hbrBackground	= (HBRUSH) GetStockObject(BLACK_BRUSH);
		wc.lpszMenuName		= 0;
		wc.lpszClassName	= _T("MAMECE_PLAYWINDOW");

		if (RegisterClass(&wc) == 0)
			return NULL;

        bRegistered = TRUE;
    }

    hWnd = CreateWindow(  _T("MAMECE_PLAYWINDOW"),
                          _T(""),
                          WS_POPUP,
                          0,
						  0,
						  GetSystemMetrics(SM_CXSCREEN),
						  GetSystemMetrics(SM_CYSCREEN),
                          MAME32App.m_hwndUI,
                          NULL,
                          hInstance,
                          NULL);

	ShowWindow(hWnd, SW_SHOWNORMAL);
	UpdateWindow(hWnd);

	return hWnd;
}

static void MAME32_ProcessMessages(void)
{
    MSG Msg;

    while (PeekMessage(&Msg, NULL, 0, 0, PM_REMOVE))
    {
        if (Msg.message == WM_QUIT)
        {
            MAME32App.Quit();
            return;
        }
        
        TranslateMessage(&Msg);
        DispatchMessage(&Msg);
    }

    return;
}

static BOOL MAME32_PumpAndReturnMessage(MSG* pMsg)
{
    if (!(GetMessage(pMsg, NULL, 0, 0)))
        return FALSE;

    TranslateMessage(pMsg);
    DispatchMessage(pMsg);

    return TRUE;
}

static void MAME32_Quit()
{
    /* Tell MAME to quit. */
    MAME32App.m_bDone = TRUE;
    //MAME32App.m_pKeyboard->PressKey(OSD_KEY_FAST_EXIT);
}

static BOOL MAME32_Done()
{
    return MAME32App.m_bDone;
}

static BOOL MAME32_Pause()
{
    if (MAME32App.m_bIsPaused)
    {
        MAME32App.m_bIsPaused = FALSE;
        MAME32App.m_bPaused = TRUE;
        return TRUE;
    }
    return FALSE;
}

static LRESULT CALLBACK MAME32_MessageProc(HWND hWnd, UINT Msg, WPARAM wParam, LPARAM lParam)
{
    LRESULT			Result = 0;

	if (MAME32App.m_pDisplay)
	    if (MAME32App.m_pDisplay->OnMessage(hWnd, Msg, wParam, lParam, &Result) == TRUE)
		    return Result;

	if (MAME32App.m_pTrak)
		if (MAME32App.m_pTrak != NULL)
			if (MAME32App.m_pTrak->OnMessage(hWnd, Msg, wParam, lParam, &Result) == TRUE)
				return Result;

	if (MAME32App.m_pKeyboard)
	    if (MAME32App.m_pKeyboard->OnMessage(hWnd, Msg, wParam, lParam, &Result) == TRUE)
		    return Result;

//	if (MAME32App.m_pSound)
//	    if (MAME32App.m_pSound->OnMessage(hWnd, Msg, wParam, lParam, &Result) == TRUE)
//		    return Result;

	if (MAME32App.m_pJoystick)
		if (MAME32App.m_pJoystick->OnMessage(hWnd, Msg, wParam, lParam, &Result) == TRUE)
			return Result;

    return DefWindowProc(hWnd, Msg, wParam, lParam);
}

static BOOL MAME32_DetectMMX()
{
	return FALSE;
}

/***************************************************************************
    Play Game
 ***************************************************************************/

options_type * GetPlayingGameOptions()
{
    return &playing_game_options;
}

int GetNumGames()
{
   return game_count;
}

game_data_type * GetGameData()
{
    return game_data;
}

/* Checks of all ROMs are available for 'game' and returns result
 * Returns TRUE if all ROMs found, 0 if any ROMs are missing.
 */
BOOL FindRomSet(int game)
{
    const struct RomModule  *romp;
    const struct GameDriver *gamedrv;

    gamedrv = drivers[game];
    romp = gamedrv->rom;

    if (!osd_faccess (gamedrv->name, OSD_FILETYPE_ROM))
    {
        /* if the game is a clone, try loading the ROM from the main version */
        if (gamedrv->clone_of == 0 ||
            !osd_faccess(gamedrv->clone_of->name,OSD_FILETYPE_ROM))
            return FALSE;
    }

    return TRUE;
}

/* Checks for all samples in a sample set.
 * Returns TRUE if all samples are found, FALSE if any are missing.
 */
BOOL FindSampleSet (int game)
{
    static const struct GameDriver *gamedrv;
    const char  *sharedname;
    BOOL bStatus;
    int  exist;
    int  skipfirst;
    int  count = 0;
    int  j;

    gamedrv = drivers[game];

    /* does the game use samples at all? */
    if (gamedrv->samplenames == 0 || gamedrv->samplenames[0] == 0)
        return TRUE;

    /* take care of shared samples */
    if (gamedrv->samplenames[0][0] == '*')
    {
        sharedname = gamedrv->samplenames[0]+1;
        skipfirst  = 1;
    }
    else
    {
        sharedname = NULL;
        skipfirst  = 0;
    }

    /* do we have samples for this game? */
    exist = osd_faccess (gamedrv->name, OSD_FILETYPE_SAMPLE);

    /* try shared samples */
    if (!exist && skipfirst)
        exist = osd_faccess (sharedname, OSD_FILETYPE_SAMPLE);

    /* if still not found, we're done */
    if (!exist)
        return FALSE;

    for (j = skipfirst; gamedrv->samplenames[j] != 0; j++)
    {
        /* skip empty definitions */
        if (strlen (gamedrv->samplenames[j]) == 0)
            continue;
        bStatus = File_Status (gamedrv->name, gamedrv->samplenames[j], OSD_FILETYPE_SAMPLE);
        if (!bStatus && skipfirst)
        {
            bStatus = File_Status (sharedname, gamedrv->samplenames[j], OSD_FILETYPE_SAMPLE);
            if (!bStatus)
            {
                return FALSE;
            }
        }
    }
    return TRUE;
}

static void MamePlayGame(int index)
{
//    RECT rect;
//    char buf[200];

    memcpy(&playing_game_options, GetGameOptions(index), sizeof(options_type));
    
    playing_game_options.sound = enable_sound ? SOUND_WAVE : SOUND_NONE;
	playing_game_options.profile = show_profiler;
	playing_game_options.showfps = show_framerate;

    /* Deal with options that can be disabled. */   
//    EnablePlayOptions(index, &playing_game_options);

    /* based on our raw options, convert the ones that we can into
       regular MAME config options */

//    if (playing_game_options.error_log)
//        options.errorlog = fopen("error.log", "wa");
//    else
        options.errorlog = NULL;

#ifdef MAME_DEBUG
    options.mame_debug = 1;
#else
    options.mame_debug = 0;
#endif
    options.cheat       = 0; // playing_game_options.cheat;
    options.gui_host    = 1;

	options.gfxopt_flip = gfxopt_flip;
	options.gfxopt_align = gfxopt_align;
	options.gfxopt_pens = gfxopt_pens;

    /*
        Need sample rate initialised _before_ rom loading
        for optional rom regions.
    */
    if (playing_game_options.sound == SOUND_NONE)
    {
        /* update the Machine structure to show that sound is disabled */
		Machine->sample_rate = 0;
		options.samplerate = 0;
    }
    else
    {
        options.samplerate  = playing_game_options.sample_rate;
    }

    options.samplebits          = playing_game_options.sample_bits;
    options.no_fm               = 1;
    options.use_samples         = playing_game_options.use_samples;
    options.use_emulated_ym3812 = (playing_game_options.fm_ym3812 == 0);
    options.norotate            = 0;  

//    switch (playing_game_options.rotate)
//    {
//    case ROTATE_RIGHT:
//        options.ror = 1;
//        options.rol = 0;
//        break;

//    case ROTATE_LEFT:
//        options.ror = 0;
//        options.rol = 1;
//        break;

//    default:
        options.ror = 0;
        options.rol = 0;
//        break;
//    }
    options.flipx = 0; // playing_game_options.flipx;
    options.flipy = 0; // playing_game_options.flipy;

    options.beam   = (int)(1 /*playing_game_options.beam*/ * 0x00010000);
    if (options.beam < 0x00010000)    /* beam MUST be at least 1 pixel */
        options.beam = 0x00010000;
    if (options.beam > 0x00100000)    /* set max to 16 pixels */
        options.beam = 0x00100000;

    options.flicker   = (int)(100 /*playing_game_options.flicker*/ * 2.55);
    if (options.flicker < 0)
        options.flicker = 0;
    if (options.flicker > 255)
        options.flicker = 255;

    options.translucency   = playing_game_options.translucency;
    options.antialias      = playing_game_options.antialias;;
    options.use_artwork    = playing_game_options.use_artwork;

//    if (joygui != NULL)
//        KillTimer(hMain,0);

//    sprintf(buf, "%s - MAME32", drivers[index]->description);
//    SetWindowText(hMain, buf);

//    abort_loading = FALSE;
//    hLoad = CreateDialog(GetModuleHandle(NULL),MAKEINTRESOURCE(IDD_LOAD_PROGRESS),hMain,
//                         LoadProgressDialogProc);

//    sprintf(buf, "Loading %s", drivers[index]->description);
//    SetWindowText(hLoad, buf);

//    GetWindowRect(hMain, &rect);

//    in_emulation = TRUE;

    if (run_game(index) == 0)
    {
//       IncrementPlayCount(index);
//       ListView_RedrawItems(hwndList,GetSelectedPickItem(),GetSelectedPickItem());
    }
    else
    {
//        if (abort_loading == FALSE)
//        { 
//            ShowWindow(hMain, SW_SHOW);
//            EndDialog(hLoad,0);
//            DialogBox(GetModuleHandle(NULL),MAKEINTRESOURCE(IDD_ROM_ERROR), NULL, LoadErrorDialogProc);
//        }
    }

//    in_emulation = FALSE;
    
//    if (quit)
//    {
//       PostMessage(hMain, WM_CLOSE, 0, 0);
//       return;
//    }


//    SetWindowText(hMain, "MAME32");
//    UpdateStatusBar();

//    ShowWindow(hMain, SW_SHOW);
//    SetFocus(hwndList);
    
//    SetWindowPos(hMain, 0, 0, 0, rect.right - rect.left,
//        rect.bottom - rect.top,
//        SWP_NOMOVE | SWP_NOZORDER | SWP_NOCOPYBITS | SWP_SHOWWINDOW);

//    if (joygui != NULL)
//        SetTimer(hMain,0,25,NULL);

//    if (playing_game_options.error_log && options.errorlog)
//        fclose(options.errorlog);
}


/***************************************************************************
    Main MAMECE Window
 ***************************************************************************/


#define MAX_LOADSTRING 100

// Global Variables:
HINSTANCE			hInst;			// The current instance
HWND				hwndCB;			// The command bar handle
HMENU				hMenu;			// The menu handle

// Foward declarations of functions included in this code module:
ATOM				MyRegisterClass	(HINSTANCE hInstance, LPTSTR szWindowClass);
BOOL				InitInstance	(HINSTANCE, int);
LRESULT CALLBACK	WndProc			(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK	About			(HWND, UINT, WPARAM, LPARAM);

int WINAPI WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow)
{
	MSG msg;
	HACCEL hAccelTable;
	int i;
	int sx, sy; 
	WORD modpath[MAX_PATH];
	char *p;

	 /* Get Module Path */
	GetModuleFileName(NULL, modpath, MAX_PATH);
	WideCharToMultiByte(CP_ACP, 0, 
		modpath, wcslen(modpath) + 1, 
		szRootPath, MAX_PATH - 1, 
		NULL, NULL);
	p = szRootPath + strlen(szRootPath);
	while (p != szRootPath && *p != '\\')
		*p-- = 0;

	/* Make default rom and sample dirs (if they don't exist) */
	CreateDirectory(WC("\\My Documents\\MameCE\\roms"), NULL);
	CreateDirectory(WC("\\My Documents\\MameCE\\samples"), NULL);

	/* Get max screen size to fix vector screen sizes with */
	sx = GetSystemMetrics(SM_CXSCREEN);
	sy = GetSystemMetrics(SM_CYSCREEN);

	/* Initialize game list */
    game_count = 0;
    while (drivers[game_count] != 0)
	{
		/* Adjust screen size for vector games so they're never larger than device screen */
		if (drivers[game_count]->drv->video_attributes & VIDEO_TYPE_VECTOR)
		{
			struct MachineDriver *machine_drv;
			int ox, oy;
			
			machine_drv = (struct MachineDriver*)ox = drivers[game_count]->drv;

			ox = machine_drv->screen_width;
			oy = machine_drv->screen_height;

			if (machine_drv->screen_width > sx)
			{
				machine_drv->screen_width = sx;
				machine_drv->screen_height = sy *
					sx / ox;
			}
			
			if (machine_drv->screen_height > sy)
			{
				machine_drv->screen_height = sy;
				machine_drv->screen_width = sx *
					sy / oy;
			}
		}

        game_count++;
	}

#ifndef NO_GAMEX
	if (!GAMEX_Init())
	{
		ErrorMsg("Your CE device doesn't support direct screen access.  "
				 "MAMECE requires a DSA compatible device to run.");
		return 1;
	}
#endif

	OptionsInit(game_count);
    File.init();

    game_data = (game_data_type *)malloc(game_count * sizeof(game_data_type));
	memset(game_data, 0, game_count * sizeof(game_data_type));
	
	game_index = (int*)malloc(game_count * sizeof(int));
	memset(game_index, 0, game_count * sizeof(int));
	game_index_count = 0;

	for (i = 0; i < game_count; i++)
	{
		game_data[i].has_roms = FindRomSet(i);
		game_data[i].has_samples = TRUE; // FindSampleSet(i);
	}

	// Perform application initialization:
	if (!InitInstance (hInstance, nCmdShow)) 
	{
		return FALSE;
	}

	hAccelTable = LoadAccelerators(hInstance, (LPCTSTR)IDC_MAMECE);

	play_game = -1;

	// Main message loop:
	while (GetMessage(&msg, NULL, 0, 0)) 
	{
		if (!TranslateAccelerator(msg.hwnd, hAccelTable, &msg)) 
		{
			TranslateMessage(&msg);
			DispatchMessage(&msg);

			if (play_game >= 0)
			{
				MamePlayGame(play_game);
				play_game = -1;
			}

		}
	}

#ifndef NO_GAMEX
	GAMEX_Close();
#endif

	return msg.wParam;
}

//
//  FUNCTION: MyRegisterClass()
//
//  PURPOSE: Registers the window class.
//
//  COMMENTS:
//
//    This function and its usage is only necessary if you want this code
//    to be compatible with Win32 systems prior to the 'RegisterClassEx'
//    function that was added to Windows 95. It is important to call this function
//    so that the application will get 'well formed' small icons associated
//    with it.
//
ATOM MyRegisterClass(HINSTANCE hInstance, LPTSTR szWindowClass)
{
	WNDCLASS	wc;

    wc.style			= CS_HREDRAW | CS_VREDRAW;
    wc.lpfnWndProc		= (WNDPROC) WndProc;
    wc.cbClsExtra		= 0;
    wc.cbWndExtra		= 0;
    wc.hInstance		= hInstance;
    wc.hIcon			= LoadIcon(hInstance, MAKEINTRESOURCE(IDI_MAMECE));
    wc.hCursor			= 0;
    wc.hbrBackground	= (HBRUSH) GetStockObject(LTGRAY_BRUSH);
    wc.lpszMenuName		= 0;
    wc.lpszClassName	= szWindowClass;

	return RegisterClass(&wc);
}

//
//  FUNCTION: InitInstance(HANDLE, int)
//
//  PURPOSE: Saves instance handle and creates main window
//
//  COMMENTS:
//
//    In this function, we save the instance handle in a global variable and
//    create and display the main program window.
//
BOOL InitInstance(HINSTANCE hInstance, int nCmdShow)
{
	HWND	hWnd;
	TCHAR	szTitle[MAX_LOADSTRING];			// The title bar text
	TCHAR	szWindowClass[MAX_LOADSTRING];		// The window class name

	hInst = hInstance;		// Store instance handle in our global variable
	// Initialize global strings
	LoadString(hInstance, IDC_MAMECE, szWindowClass, MAX_LOADSTRING);
	MyRegisterClass(hInstance, szWindowClass);

	LoadString(hInstance, IDS_APP_TITLE, szTitle, MAX_LOADSTRING);
	hWnd = CreateWindow(szWindowClass, szTitle, WS_VISIBLE,
		0, 0, CW_USEDEFAULT, CW_USEDEFAULT, NULL, NULL, hInstance, NULL);

	if (!hWnd)
	{	
		return FALSE;
	}

	ShowWindow(hWnd, nCmdShow);
	UpdateWindow(hWnd);
	if (hwndCB)
		CommandBar_Show(hwndCB, TRUE);

	return TRUE;
}

//
//  FUNCTION: WndProc(HWND, unsigned, WORD, LONG)
//
//  PURPOSE:  Processes messages for the main window.
//
//  WM_COMMAND	- process the application menu
//  WM_PAINT	- Paint the main window
//  WM_DESTROY	- post a quit message and return
//
//
LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	HDC hdc;
	int wmId, wmEvent;
	PAINTSTRUCT ps;
	TCHAR szHello[MAX_LOADSTRING];
	RECT rt;
	int game;

	switch (message) 
	{
		case WM_COMMAND:
			wmId    = LOWORD(wParam); 
			wmEvent = HIWORD(wParam); 
			// Parse the menu selections:
			switch (wmId)
			{
				case IDM_HELP_ABOUT:
				   DialogBox(hInst, (LPCTSTR)IDD_ABOUTBOX, hWnd, (DLGPROC)About);
				   break;
				case IDM_FILE_EXIT:
				   DestroyWindow(hWnd);
				   break;
				case IDM_OPTIONS_SOUND:
				   enable_sound = !enable_sound;
				   CheckMenuItem(hMenu, IDM_OPTIONS_SOUND, 
						MF_BYCOMMAND | (enable_sound ? MF_CHECKED : MF_UNCHECKED) );
				   break;
				case IDM_OPTIONS_PROFILER:
				   show_profiler = !show_profiler;
				   CheckMenuItem(hMenu, IDM_OPTIONS_PROFILER,
						MF_BYCOMMAND | (show_profiler ? MF_CHECKED : MF_UNCHECKED) );
				   break;
				case IDM_OPTIONS_FRAMERATE:
				   show_framerate = !show_framerate;
				   CheckMenuItem(hMenu, IDM_OPTIONS_FRAMERATE,
						MF_BYCOMMAND | (show_framerate ? MF_CHECKED : MF_UNCHECKED) );
				   break;
				case IDM_OPTIONS_SHOWALLGAMES:
				   show_all_games = !show_all_games;
				   CheckMenuItem(hMenu, IDM_OPTIONS_SHOWALLGAMES,
						MF_BYCOMMAND | (show_all_games ? MF_CHECKED : MF_UNCHECKED) );
				   RefreshGameListBox();
				   break;
				case IDM_OPTIONS_SHOWCLONES:
				   show_clones = !show_clones;
				   CheckMenuItem(hMenu, IDM_OPTIONS_SHOWCLONES, 
						MF_BYCOMMAND | (show_clones ? MF_CHECKED : MF_UNCHECKED) );
				   RefreshGameListBox();
				   break;
				case IDM_OPTIONS_OPTIMIZEFLIPPING:
				   gfxopt_flip = !gfxopt_flip;
				   CheckMenuItem(hMenu, IDM_OPTIONS_OPTIMIZEFLIPPING,
						MF_BYCOMMAND | (gfxopt_flip ? MF_CHECKED : MF_UNCHECKED) );
				   break;
				case IDM_OPTIONS_OPTIMIZEALIGNMENT:
				   gfxopt_align = !gfxopt_align;
				   CheckMenuItem(hMenu, IDM_OPTIONS_OPTIMIZEALIGNMENT,
						MF_BYCOMMAND | (gfxopt_align ? MF_CHECKED : MF_UNCHECKED) );
				   break;
				case IDM_OPTIONS_OPTIMIZEPENS:
				   gfxopt_pens = !gfxopt_pens;
				   CheckMenuItem(hMenu, IDM_OPTIONS_OPTIMIZEPENS,
						MF_BYCOMMAND | (gfxopt_pens ? MF_CHECKED : MF_UNCHECKED) );
				   break;

				case IDC_PLAY:
				   if (wmEvent == BN_CLICKED)
					   play_game = SendMessage(hGameList, LB_GETCURSEL, 0, 0);
					   if (play_game >= 0)
						   play_game = game_index[play_game];
				   break;
				case IDC_PLAYLIST:
				   if (wmEvent == LBN_DBLCLK)
					   play_game = SendMessage(hGameList, LB_GETCURSEL, 0, 0);
					   if (play_game >= 0)
						   play_game = game_index[play_game];
				   break;
				case IDC_OPTIONS:
				   break;
			
				default:
				   return DefWindowProc(hWnd, message, wParam, lParam);
			}
			break;
		case WM_CREATE:
			hwndCB = CommandBar_Create(hInst, hWnd, 1);			
			CommandBar_InsertMenubar(hwndCB, hInst, IDM_MENU, 0);
			CommandBar_AddAdornments(hwndCB, 0, 0);
			hMenu = CommandBar_GetMenu(hwndCB, 0); 

			GetClientRect(hWnd, &rt);
			hGameList = CreateWindowEx(0, _T("LISTBOX"), _T(""),
				WS_VISIBLE | WS_CHILD | WS_TABSTOP | LBS_STANDARD | LBS_NOINTEGRALHEIGHT, 
				4, 30, rt.right - 8, rt.bottom - 38 - 30,
				hWnd, (HMENU)IDC_PLAYLIST, hInst, NULL);
			CreateWindowEx(0, _T("BUTTON"), _T("Play"),
				WS_VISIBLE | WS_CHILD | WS_TABSTOP, 
				4, rt.bottom - 30 - 4, 114, 30, 
				hWnd, (HMENU)IDC_PLAY, hInst, NULL); 
			CreateWindowEx(0, _T("BUTTON"), _T("Options"),
				WS_VISIBLE | WS_CHILD | WS_TABSTOP, 
				122, rt.bottom - 30 - 4, 114, 30, 
				hWnd, (HMENU)IDC_OPTIONS, hInst, NULL); 

			RefreshGameListBox();

			break;
		case WM_DESTROY:
			CommandBar_Destroy(hwndCB);
			PostQuitMessage(0);
			break;
		default:
			return DefWindowProc(hWnd, message, wParam, lParam);
   }
   return 0;
}

void RefreshGameListBox()
{
	int i, j, pos;

	SendMessage(hGameList, LB_RESETCONTENT, 0, 0);

	game_index_count = 0;
	for (i = 0; i < game_count; i++)
	{
		if (!show_all_games && !game_data[i].has_roms)
			continue;

		if (!show_all_games && !show_clones && drivers[i]->clone_of != NULL)
			continue;

		pos = SendMessage(hGameList, LB_ADDSTRING, 
			0, (WPARAM)WC(drivers[i]->description));

		for (j = game_index_count - 1; j >= pos; j--)
			game_index[j + 1] = game_index[j];
		game_index[pos] = i;
		game_index_count++;
	}
}

// Mesage handler for the About box.
LRESULT CALLBACK About(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	RECT rt, rt1;
	int DlgWidth, DlgHeight;	// dialog width and height in pixel units
	int NewPosX, NewPosY;

	switch (message)
	{
		case WM_INITDIALOG:
			// trying to center the About dialog
			if (GetWindowRect(hDlg, &rt1)) {
				GetClientRect(GetParent(hDlg), &rt);
				DlgWidth	= rt1.right - rt1.left;
				DlgHeight	= rt1.bottom - rt1.top ;
				NewPosX		= (rt.right - rt.left - DlgWidth)/2;
				NewPosY		= (rt.bottom - rt.top - DlgHeight)/2;
				
				// if the About box is larger than the physical screen 
				if (NewPosX < 0) NewPosX = 0;
				if (NewPosY < 0) NewPosY = 0;
				SetWindowPos(hDlg, 0, NewPosX, NewPosY,
					0, 0, SWP_NOZORDER | SWP_NOSIZE);
			}
			return TRUE;

		case WM_COMMAND:
			if ((LOWORD(wParam) == IDOK) || (LOWORD(wParam) == IDCANCEL))
			{
				EndDialog(hDlg, LOWORD(wParam));
				return TRUE;
			}
			break;
	}
    return FALSE;
}
